-----------------------------------------------------------------------------
-- HTK
-----------------------------------------------------------------------------
-- A toolkit made with htmltoolkit.
-- HTK offers a collection of constructors very similar to HTML elements,
-- but with "Lua style".
-----------------------------------------------------------------------------

$ifnot HTK

HTK = {}

-----------------------------------------------------------------------------
-- VIRTUAL
-- Constructors that encapsulate HTML elements giving a "Lua style":
--	FRAME
--	MULTILINE
-----------------------------------------------------------------------------
function HTK.FRAME (param)
   local t = {}
   if param.border then
      t.border = param.border
      param.border = nil
   else
      t.border = 1
   end
   t[1] = TR { TD (param) }
   return TABLE (t)
end

function HTK.MULTILINE (param)
   if param.value then
      param[1] = param.value
      param.value = nil
   end
   return TEXTAREA (param)
end

-----------------------------------------------------------------------------
-- INPUT
-- Constructors that encapsulate the INPUT element:
--	BUTTON
--	HIDDEN
--	PASSWORD
--	RADIO_BUTTON == INPUT with type='radio'
--	RESET
--	SUBMIT
--	TEXT
--	TOGGLE == INPUT with type='checkbox'
-----------------------------------------------------------------------------
function HTK.BUTTON (param)
   param.type = "button"
   return INPUT (param)
end

function HTK.HIDDEN (param)
   param.type = "hidden"
   return INPUT (param)
end

function HTK.PASSWORD (param)
   param.type = "password"
   return INPUT (param)
end

function HTK.RADIO_BUTTON (param)
   param.type = "radio"
   return INPUT (param)
end

function HTK.RESET (param)
   param.type = "reset"
   return INPUT (param)
end

function HTK.SUBMIT (param)
   param.type = "submit"
   return INPUT (param)
end

function HTK.TEXT (param)
   param.type = "text"
   return INPUT (param)
end

function HTK.TOGGLE (param)
   param.type = "checkbox"
   return INPUT (param)
end

-----------------------------------------------------------------------------
-- CONTAINERS
-- Constructors that creates some abstractions or encapsulation:
--	RADIO (many RADIO_BUTTON's == INPUT with type='radio')
--	TOGGLE_LIST (many TOGGLE's == INPUT with type='checkbox')
--	SELECT (manu OPTION's enclosed in a SELECT element)
-----------------------------------------------------------------------------
function HTK._button_list (param, element)
   local i = 1
   while param[i] do
      if type(param[i]) == "table" then
         if not param[i].name then
            param[i].name = param.name
         end
         if not param[i].value then
            param[i].value = param[i][1]
         end
         param[i] = BOX {
		HTK[element] (param[i]),
		BR {},
         }
      else
         param[i] = BOX {
		HTK[element] { name = param.name, value = param[i], },
		param[i],
		BR {},
         }
      end
      i = i+1
   end
   param.separator = '\n'
   return BOX (param)
end

function HTK.RADIO (param)
   return HTK._button_list (param, "RADIO_BUTTON")
end

function HTK.TOGGLE_LIST (param)
   return HTK._button_list (param, "TOGGLE")
end

function HTK.SELECT (param)
   local i = 1
   while param[i] do
      if type(param[i]) == "table" then
         param[i] = OPTION { param[i][1]; value = param[i].value,
		selected = param[i].selected }
      else
         param[i] = OPTION { param[i]; selected = param.selected==i }
      end
      i = i+1
   end
   param.separator = '\n'
   param.selected = nil
   return SELECT (param)
end

-----------------------------------------------------------------------------
-- COMPOUND Elements
-- Constructors that join together some elements to create a more powerfull
-- abstraction:
--	-- RADIO + TEXT => ( ) 1
--			   ( ) 2
--			   ( ) Mais de 2 _____________________________
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- TABLE Cells
-- Constructors that "give" a face and size attributes, to set fonts.
-----------------------------------------------------------------------------
function HTK._cell (param)
   if param.face then
      local content = { separator='\n', face=param.face, size=param.size, }
      local i = 1
      while param[i] do
         content[i] = param[i]
         param[i] = nil
         i = i+1
      end
      param[1] = FONT (content)
      param.face = nil
      param.size = nil
   end
   return param
end

function HTK.TH (param)
   return TH (HTK._cell(param))
end

function HTK.TD (param)
   return TD (HTK._cell(param))
end

-----------------------------------------------------------------------------
-- TABLE Rows
-- Constructors that creates "fields" as table rows with two cells, the
-- left one with a label and the right one with an input element.
--	MULTILINE_FIELD
--	PASSWORD_FIELD
--	RADIO_FIELD
--	SELECT_FIELD
--	TEXT_FIELD
--	TEXTAREA_FIELD
--	TOGGLE_FIELD
-----------------------------------------------------------------------------
function HTK._field (param, element)
   local label = param.label
   local font_size = param.font_size
   param.label = nil
   param.font_size = nil
   return TR {
	HTK.TD { label; face = param.face, size = font_size },
	HTK.TD { HTK[element] (param); face = param.face, size = font_size },
	; separator = '\n',
   }
end

function HTK.PASSWORD_FIELD (param)
   return HTK._field (param, "PASSWORD")
end

function HTK.RADIO_FIELD (param)
   return HTK._field (param, "RADIO")
end

function HTK.SELECT_FIELD (param)
   return HTK._field (param, "SELECT")
end

function HTK.TEXT_FIELD (param)
   return HTK._field (param, "TEXT")
end

function HTK.TEXTAREA_FIELD (param)
   return HTK._field (param, "MULTILINE")
end

HTK.MULTILINE_FIELD = HTK.TEXTAREA_FIELD

function HTK.TOGGLE_FIELD (param)
   return HTK._field (param, "TOGGLE")
end

$end
